var JumbledObserver = Class.create({
  initialize: function(element, observer) {
    this.element   = $(element);
    this.observer  = observer;
    this.lastValue = Jumbled.serialize(this.element);
  },
  
  onStart: function() {
    this.lastValue = Jumbled.serialize(this.element);
    //alert(this.element.handle);
    drag_active = true;
  },
    
  onEnd: function() {
    Jumbled.unmark();
    if(this.lastValue != Jumbled.serialize(this.element))
      this.observer(this.element)
    drag_active = false;
  }
});

var Jumbled = {};
Object.extend(Jumbled, 
    Object.extend(Sortable, {
        create: function(element) {
        element = $(element);
        var options = Object.extend({ 
            element:     element,
            tag:         'li',       // assumes li children, override with tag: 'tagname'
            dropOnEmpty: false,
            tree:        false,
            treeTag:     'ul',
            overlap:     'vertical', // one of 'vertical', 'horizontal'
            constraint:  'vertical', // one of 'vertical', 'horizontal', false
            containment: element,    // also takes array of elements (or id's); or false
            handle:      false,      // or a CSS class
            only:        false,
            delay:       0,
            hoverclass:  null,
            ghosting:    false,
            quiet:       false, 
            scroll:      false,
            scrollSensitivity: 20,
            scrollSpeed: 15,
            format:      this.SERIALIZE_RULE,
      
            // these take arrays of elements or ids and can be 
            // used for better initialization performance
            elements:    false,
            handles:     false,
      
            onChange:    Prototype.emptyFunction,
            onUpdate:    Prototype.emptyFunction
        }, arguments[1] || { });

        // clear any old sortable with same element
        this.destroy(element);

        // build options for the draggables
        var options_for_draggable = {
            revert:      true,
            quiet:       options.quiet,
            scroll:      options.scroll,
            scrollSpeed: options.scrollSpeed,
            scrollSensitivity: options.scrollSensitivity,
            delay:       options.delay,
            ghosting:    options.ghosting,
            constraint:  options.constraint,
            handle:      options.handle};

        if(options.starteffect)
            options_for_draggable.starteffect = options.starteffect;

        if(options.reverteffect)
            options_for_draggable.reverteffect = options.reverteffect;
        else
        if(options.ghosting) options_for_draggable.reverteffect = function(element) {
            element.style.top  = 0;
            element.style.left = 0;
        };

        if(options.endeffect)
            options_for_draggable.endeffect = options.endeffect;

        if(options.zindex)
            options_for_draggable.zindex = options.zindex;

        // build options for the droppables  
        var options_for_droppable = {
            overlap:     options.overlap,
            containment: options.containment,
            tree:        options.tree,
            hoverclass:  options.hoverclass,
            onHover:     Sortable.onHover
        }
    
        var options_for_tree = {
            onHover:      Sortable.onEmptyHover,
            overlap:      options.overlap,
            containment:  options.containment,
            hoverclass:   options.hoverclass
        }

        // fix for gecko engine
        Element.cleanWhitespace(element); 

        options.draggables = [];
        options.droppables = [];

        // drop on empty handling
        if(options.dropOnEmpty || options.tree) {
            Droppables.add(element, options_for_tree);
            options.droppables.push(element);
        }

        (options.elements || this.findElements(element, options) || []).each( function(e,i) {
            var handle = options.handles ? $(options.handles[i]) :
                (options.handle ? $(e).select('.' + options.handle)[0] : e); 
            options.draggables.push(
                new Draggable(e, Object.extend(options_for_draggable, { handle: handle })));
            Droppables.add(e, options_for_droppable);
            if(options.tree) e.treeNode = element;
                options.droppables.push(e);      
        });
    
        if(options.tree) {
            (Sortable.findTreeElements(element, options) || []).each( function(e) {
                Droppables.add(e, options_for_tree);
                e.treeNode = element;
                options.droppables.push(e);
            });
        }

        // keep reference
        this.sortables[element.id] = options;

        // for onupdate
        Draggables.addObserver(new JumbledObserver(element, options.onUpdate));

    },
        onHover: function(element,dropon,overlap) {            
            if(Element.isParent(dropon, element)) return;
            var next = element.nextSibling;            
            
            if(overlap>0.5) {
                Sortable.mark(dropon, 'before');
                if(dropon.previousSibling != element) {
                    element.style.visibility = "hidden"; // fix gecko rendering
                    dropon.parentNode.insertBefore(element, dropon);                    
                    Sortable.options(dropon.parentNode).onChange(element);
                }
            } else {
                Sortable.mark(dropon, 'after');
                var nextElement = dropon.nextSibling || null;
                if(nextElement != element) {
                    element.style.visibility = "hidden"; // fix gecko rendering
                    dropon.parentNode.insertBefore(element, nextElement);                    
                    Sortable.options(dropon.parentNode).onChange(element);
                }
            }

            if (next)  {          
                next.parentNode.insertBefore(dropon, next);                
                Sortable.options(dropon.parentNode).onChange(next);
            } else {
                element.parentNode.appendChild(dropon);
                Sortable.options(dropon.parentNode).onChange(element);
            }
        },
        findElements: function(element, options) {
            var children = Element.findChildren(element, options.only, options.tree ? true : false, options.tag);
            var new_children = new Array();
            
            children.each(function(n) {
                if (n.className.indexOf('correct')<0)
                    new_children.push(n);                
            });
            return new_children;
        }
    })
);
    
function error_handler(t) {
    alert('Fehler: ' + t.status + ' -- ' + t.statusText);
}

function reset_items(element) {
    var element = $(element);
    for (var i=0;i<element.childNodes.length;i++) {
        Element.removeClassName($(element.childNodes[i].id),"evaluating");
        Element.removeClassName($(element.childNodes[i].id),"correct");
        Element.removeClassName($(element.childNodes[i].id),"wrong");
    }
}


function reset_field(fields,dir,css) {
    for (var i=0;i<fields.length;i++) {
        if (fields[i]=="") break;
        reset_items("list_"+fields[i]);
        Jumbled.create('list_'+fields[i], {overlap:dir,constraint:dir,only:css,onUpdate:eval(on_update)});
    }
}

function reset_page() {
    var horizontal_on_page=$("page_"+current_page+"_horizontal").value.split(",");
    var vertical_on_page=$("page_"+current_page+"_vertical").value.split(",");
    var letters_on_page=$("page_"+current_page+"_letters").value.split(",");
    var pictures_on_page=$("page_"+current_page+"_pictures").value.split(",");
    reset_field(letters_on_page,"horizontal", "jumbledletters_li_dnd");
    reset_field(horizontal_on_page,"horizontal", "jumbledwords_li_dnd");
    reset_field(vertical_on_page,"vertical", "jumbledlines_li_dnd");
    reset_field(pictures_on_page,"horizontal", "jumbledpictures_li_dnd");
}

function reset_container_items(container) {
    reset_items(container.id);
}

function my_serialize(element) {
    var element = $(element);
    res=""
    reset_items(element);
    for (var i=0;i<element.childNodes.length;i++) {
        if (element.childNodes[i].id) {
            if (res.length!=0) res=res+","
                res=res+element.childNodes[i].id;
            Element.addClassName($(element.childNodes[i].id),"evaluating")
        }
    }
    return res;
}
            
function jumbled_updated(container) {    
    var request_url=window.location.href;
    var ampi="&amp;"
    ampi=ampi.substr(0,1);
    
    var ids = container.id+"="+my_serialize(container.id);
        
    request_url=request_url.replace(/\/view\/.*/,"/validate_id/");
    request_url=request_url.replace(/\?.*/,"");                    
    new Ajax.Request(request_url,{method:'pos',postBody:"page_id="+current_page+ampi+'jumbled_id='+container.id+ampi+ids,onSuccess:success_handler_jumbled,onFailure:error_handler});
}

function recreate_jumbled(container) {
    var container_id = container.id;
    var container_type = container.className;
    
    if (container_type == "jumbledlines") {
        Jumbled.create(container_id, {only:"jumbledlines_li_dnd",onChange:jumbled_lines_changed,onUpdate:jumbled_updated});                            
    } else if (container_type == "jumbledwords") {
        Jumbled.create(container_id, {overlap:"horizontal",constraint:"horizontal",only:"jumbledwords_li_dnd",onUpdate:jumbled_updated});
    } else if (container_type == "jumbledletters") {
        Jumbled.create(container_id, {overlap:"horizontal",constraint:"horizontal",only:"jumbledletters_li_dnd",onUpdate:jumbled_updated});
    } else {
        Jumbled.create(container_id, {overlap:"horizontal",constraint:"horizontal",only:"jumbledpictures_li_dnd",onUpdate:jumbled_updated});
    }    
}

function success_handler_jumbled(t) {
    var data=eval("("+t.responseText+")");                   
    var res=data.results;
                            
    for (var i=0;i<res.length;i++) {
        Element.removeClassName(res[i].id,'evaluating');
        Element.removeClassName(res[i].id,'correct');
        Element.removeClassName(res[i].id,'wrong');
        if (res[i].result==true) {
            Element.addClassName(res[i].id,'correct');            
        }
        hide_solution(res[i].id);            
        // Rot ist zu aggressiv - gelb besser bei sofortiger auswertung (nicht bei I'm done).
        /* else {
            Element.addClassName(res[i].id,'wrong');
        }*/
    }
    
    if (res) {
        recreate_jumbled($(res[0].id).parentNode);
    }
    
    
}
            
function get_and_disable(fields) {
    tmp=""
    for(var i=0;i<fields.length;i++) {
        if (fields[i]=="") break;
            Jumbled.destroy("list_"+fields[i]);
            tmp+="list_"+fields[i]+"="+my_serialize("list_"+fields[i])+"&";
        }
    return tmp;
}

function evaluate_form(callback) {
    drag_active = false;
    var postBody="";
    var ampi="&amp;"
    ampi=ampi.substr(0,1);
    postBody=postBody+="page_id="+current_page+ampi;
    var horizontal_on_page=$("page_"+current_page+"_horizontal").value.split(",");                                
    var vertical_on_page=$("page_"+current_page+"_vertical").value.split(",");                
    var letters_on_page=$("page_"+current_page+"_letters").value.split(",");
    var pictures_on_page=$("page_"+current_page+"_pictures").value.split(",");
                
    postBody+=get_and_disable(horizontal_on_page);
    postBody+=get_and_disable(vertical_on_page);
    postBody+=get_and_disable(letters_on_page);
    postBody+=get_and_disable(pictures_on_page);
    /*window.alert(postBody);*/
    var request_url=window.location.href;
    request_url=request_url.replace(/\/view\/.*/,"/evaluate_form/");
    request_url=request_url.replace(/\?.*/,"");
    new Ajax.Request(request_url,{method:'post',postBody:postBody,onSuccess:(callback?callback:setup_success_handler_task(success_handler_form)),onFailure:error_handler});
}

function success_handler_form(t) {
    Element.hide("hint_div");
    var data=eval("("+t.responseText+")");                   
                
    var res=data.results;
    for (var i=0;i<res.length;i++) {
        Element.removeClassName(res[i].id,'evaluating');
        if (res[i].result==true) {
            Element.addClassName(res[i].id,'correct');            
        } else {
            Element.addClassName(res[i].id,'wrong');
        }
    }
}

function update_list_heights(list1, list2) {
    var i;
        
    if ($(list1))
        var list1_elements = $(list1).getElementsByTagName('li');
    else
        return;
        
    if ($(list2))
        var list2_elements = $(list2).getElementsByTagName('li');
    else
        return;    
        
    for (i=0; i<list1_elements.length; i++) {
        list1_elements[i].style.height="";
        list2_elements[i].style.height="";
        if (list1_elements[i].offsetHeight != list2_elements[i].offsetHeight)
            if (list1_elements[i].offsetHeight > list2_elements[i].offsetHeight) {
                list1_elements[i].style.height=list1_elements[i].offsetHeight-2+"px";
                list2_elements[i].style.height=list1_elements[i].offsetHeight-2+"px";
            } else {
                list1_elements[i].style.height=list2_elements[i].offsetHeight-2+"px"; 
                list2_elements[i].style.height=list2_elements[i].offsetHeight-2+"px"; 
            }
    }            
}

function fix_jumbled_lines_width() {
    var id_regexp = /list_(\d+)_left/;
    var elements = Element.getElementsBySelector("task_content_"+current_page, "ul.jumbledlines");
    var ids = new Array();
    
    for (var i=0; i<elements.length; i++) {
        if (id_regexp.exec(elements[i].id))
            ids.push("list_"+RegExp.$1);        
    }
    
    for (var i=0; i<ids.length; i++) {
        try {
            var list1_elements = $(ids[i]+"_left").getElementsByTagName('li');
            for (j=0; j<list1_elements.length; j++) {
                list1_elements[j].style.width = list1_elements[0].offsetWidth-2+"px";
            }
        } catch (e) {}
        
        try {
            var list2_elements = $(ids[i]+"_right").getElementsByTagName('li');                                   
            for (j=0; j<list2_elements.length; j++)
                list2_elements[j].style.width = list2_elements[0].offsetWidth-2+"px";
        } catch (e) {}                    
    }
}

function fix_jumbled_lines_height() {
    var id_regexp = /list_(\d+)_left/;
    var elements = Element.getElementsBySelector("task_content_"+current_page, "ul.jumbledlines");
    var ids = new Array();
    
    for (var i=0; i<elements.length; i++) {
        if (id_regexp.exec(elements[i].id))
            ids.push("list_"+RegExp.$1);        
    }
    
    for (var i=0; i<ids.length; i++) {
        update_list_heights(ids[i]+"_left", ids[i]+"_right");
    }            
}

function jumbled_lines_changed(element) {
    var id_regexp = /list_(\d+)_\w+/;
    id_regexp.exec(element.parentNode.id);
    var container_id = RegExp.$1;
    update_list_heights("list_"+container_id+"_left", "list_"+container_id+"_right");
}

function register_solution(element_id, equalize_width, equalize_height) {
    var element = $(element_id);
    element.observe('mouseover', function(event) {
        show_solution(element_id, equalize_width, equalize_height);
    });
    element.observe('mouseout', function(event) {
        hide_solution(element_id, equalize_width, equalize_height);
    });
}    

function show_solution(element_id, equalize_width, equalize_height) {
    if (drag_active)
        return;
        
    var data = $(element_id+'_data');
    var solution = $(element_id+'_solution');
    
    // No solutions for evaluated elements
    
    if ($(element_id).className.indexOf('wrong')>=0)
        return;
        
    if ($(element_id).className.indexOf('correct')>=0)
        return;    
        
    if (solution)
        if (!Element.visible(solution)) {
            if (equalize_width)
                if (!solution.style.width && !data.style.width)
                    if (solution.getWidth() < data.getWidth())
                        solution.style.width=data.getWidth()-10+"px";
                    else
                        data.style.width=solution.getWidth()-10+"px";      // 10 = padding_left + padding_right                                  
            if (equalize_height)
                if (!solution.style.height && !data.style.height)
                    if (solution.getHeight() < data.getHeight()) {
                        solution.style.height=data.getHeight()+"px";
                        solution.style.lineHeight=data.getHeight()+"px";
                    } else {
                        data.style.height=solution.getHeight()+"px";      // 10 = padding_left + padding_right                                              
                        data.style.lineHeight=solution.getHeight()+"px";                    
                    }    
            data.hide();
            Element.addClassName(element_id, "solution");
            solution.show();
        }    
}

function hide_solution(element_id) {
    if (drag_active)
        return;
                
    var data = $(element_id+'_data');
    var solution = $(element_id+'_solution');
    
    if (solution)
        if (Element.visible(solution)) {   
            solution.hide();        
            Element.removeClassName(element_id, "solution");
            data.show();
        }    
}


function try_set() {
    if (window.location.hash!='#fillin_correct') {
        var request_url=window.location.href;
        request_url=request_url.replace(/\/view\/.*/,"/reset_task/");
        request_url=request_url.replace(/\?.*/,"");
        new Ajax.Request(request_url,{method:'post',postBody:"reset_it=1",onSuccess:function(){},onFailure:error_handler});
    } else {
        get_correct();
    }
    fix_jumbled_lines_height();        
    fix_jumbled_lines_width();
}

window.onload=try_set;


function get_correct() {
    var request_url=window.location.href;
    request_url=request_url.replace(/\/view\/.*/,"/get_correct/");
    request_url=request_url.replace(/\?.*/,"");
    new Ajax.Request(request_url,{method:'post',postBody:"get_it="+(new Date()),onSuccess:success_handler_get_correct,onFailure:error_handler});
}


function success_handler_get_correct(t) {
    Element.hide("hint_div");
//     window.alert(t.responseText);
// try {
    var data=eval("("+t.responseText+")");                   

    var pos=data.positions;
    for (var i=0;i<pos.length;i++) {
        var id=pos[i].id
        Jumbled.destroy("list_"+id);
        reset_items("list_"+id);
        var order=pos[i].order.split(",")
        var o_items=[]
        for (var o=0;o<order.length;o++) {
            if (order[o]=="") continue;
            o_items.push($(order[o]));
        }

//         window.alert(o_items);
        var p=$("list_"+id);
        for (var j=0;j<o_items.length;j++) {
            p.appendChild(o_items[j]);
        }
//         window.alert(all_jumbled_containers["list_"+id]);
        Jumbled.create('list_'+id,{overlap:all_jumbled_containers["list_"+id].dir,constraint:all_jumbled_containers["list_"+id].dir,only:all_jumbled_containers["list_"+id].css,onUpdate:eval(on_update)});
    }

    var res=data.results;
//     window.alert(res);
    for (var i=0;i<res.length;i++) {
        Element.removeClassName(res[i].id,'evaluating');
        if (res[i].result==true) {
            Element.addClassName(res[i].id,'correct');
        } else {
            //Element.addClassName(res[i].id,'wrong');
        }
    }

// }catch (e) {window.alert(e);}
}

function page_view() {
    fix_jumbled_lines_height();
    fix_jumbled_lines_width();
}